@@ -43,7 +43,7 @@ class BetterGettableParam(Parameter):
 
     def get_raw(self):
         self._get_count += 1
-        return self._latest['raw_value']
+        return self.cache._raw_value
 
 
 class DeprecatedParam(Parameter):
@@ -99,7 +99,7 @@ class MemoryParameter(Parameter):
             if func is not None:
                 val = func()
             else:
-                val = self._latest['raw_value']
+                val = self.cache._raw_value
             self.get_values.append(val)
             return val
         return get_func
@@ -201,6 +201,21 @@ class TestParameter(TestCase):
         self.assertEqual(p.vals.values_validated,
                          [0, 0, 10, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10])
 
+    def test_number_of_validations_for_set_cache(self):
+        p = Parameter('p', set_cmd=None,
+                      vals=BookkeepingValidator())
+        self.assertEqual(p.vals.values_validated, [])
+
+        p.cache.set(1)
+        self.assertEqual(p.vals.values_validated, [1])
+
+        p.cache.set(4)
+        self.assertEqual(p.vals.values_validated, [1, 4])
+
+        p.step = 1
+        p.cache.set(10)
+        self.assertEqual(p.vals.values_validated, [1, 4, 10])
+
     def test_snapshot_value(self):
         p_snapshot = Parameter('no_snapshot', set_cmd=None, get_cmd=None,
                                snapshot_value=True)
@@ -235,6 +250,24 @@ class TestParameter(TestCase):
         self.assertEqual(local_parameter.get_latest(), 2)
         self.assertGreater(local_parameter.get_latest.get_timestamp(), after_set)
 
+    def test_get_latest_raw_value(self):
+        # To have a simple distinction between raw value and value of the
+        # parameter lets create a parameter with an offset
+        p = Parameter('p', set_cmd=None, get_cmd=None, offset=42)
+        assert p.get_latest.get_timestamp() is None
+
+        # Initially, the parameter's raw value is None
+        assert p.get_latest.get_raw_value() is None
+
+        # After setting the parameter to some value, the
+        # ``.get_latest.get_raw_value()`` call should return the new raw value
+        # of the parameter
+        p(3)
+        assert p.get_latest.get_timestamp() is not None
+        assert p.get_latest.get() == 3
+        assert p.get_latest() == 3
+        assert p.get_latest.get_raw_value() == 3 + 42
+
     def test_get_latest_unknown(self):
         """
         Test that get latest on a parameter that has not been acquired will
@@ -245,8 +278,8 @@ class TestParameter(TestCase):
                                               get_cmd=None)
         # fake a parameter that has a value but never been get/set to mock
         # an instrument.
-        local_parameter._latest = {"value": value, "raw_value": value,
-                                   'ts': None}
+        local_parameter.cache._value = value
+        local_parameter.cache._raw_value = value
         assert local_parameter.get_latest.get_timestamp() is None
         before_get = datetime.now()
         assert local_parameter._get_count == 0
@@ -271,8 +304,8 @@ class TestParameter(TestCase):
         # fake a parameter that has a value acquired 10 sec ago
         start = datetime.now()
         set_time = start - timedelta(seconds=10)
-        local_parameter._latest = {"value": value, "raw_value": value,
-                                   'ts': set_time}
+        local_parameter.cache._update_with(
+            value=value, raw_value=value, timestamp=set_time)
         assert local_parameter._get_count == 0
         assert local_parameter.get_latest.get_timestamp() == set_time
         assert local_parameter.get_latest() == value
@@ -311,14 +344,14 @@ class TestParameter(TestCase):
                                               set_cmd=None,
                                               max_val_age=1,
                                               initial_value=value)
-        assert local_parameter.get_latest.max_val_age == 1
+        assert local_parameter.cache.max_val_age == 1
         assert local_parameter._get_count == 0
         assert local_parameter.get_latest() == value
         assert local_parameter._get_count == 0
         # now fake the time stamp so get should be triggered
         set_time = start - timedelta(seconds=10)
-        local_parameter._latest = {"value": value, "raw_value": value,
-                                   'ts': set_time}
+        local_parameter.cache._update_with(
+            value=value, raw_value=value, timestamp=set_time)
         # now that ts < max_val_age calling get_latest should update the time
         assert local_parameter.get_latest.get_timestamp() == set_time
         assert local_parameter.get_latest() == value
@@ -340,10 +373,9 @@ class TestParameter(TestCase):
         # in a subclass. Here we create a subclass that does add a get command and alsoo does 
         # not implement the check for max_val_age
         class LocalParameter(_BaseParameter):
-
             def __init__(self, *args, **kwargs):
                 super().__init__(*args, **kwargs)
-                self.set_raw = partial(self._save_val, validate=False)
+                self.set_raw = lambda x: x
                 self.set = self._wrap_set(self.set_raw)
 
         localparameter = LocalParameter('test_param',
@@ -353,6 +385,18 @@ class TestParameter(TestCase):
         with self.assertRaises(RuntimeError):
             localparameter.get_latest()
 
+    def test_latest_dictionary_gets_updated_upon_set_of_memory_parameter(self):
+        p = Parameter('p', set_cmd=None, get_cmd=None)
+        assert p.cache._value is None
+        assert p.cache._raw_value is None
+        assert p.cache.timestamp is None
+
+        p(42)
+
+        assert p.cache._value == 42
+        assert p.cache._raw_value == 42
+        assert p.cache.timestamp is not None
+
     def test_has_set_get(self):
         # Create parameter that has no set_cmd, and get_cmd returns last value
         gettable_parameter = Parameter('one', set_cmd=False, get_cmd=None)
@@ -362,6 +406,9 @@ class TestParameter(TestCase):
             gettable_parameter(1)
         # Initial value is None if not explicitly set
         self.assertIsNone(gettable_parameter())
+        # Assert the ``cache.set`` still works for non-settable parameter
+        gettable_parameter.cache.set(1)
+        self.assertEqual(gettable_parameter(), 1)
 
         # Create parameter that saves value during set, and has no get_cmd
         settable_parameter = Parameter('two', set_cmd=None, get_cmd=False)
@@ -414,6 +461,13 @@ class TestParameter(TestCase):
         p(44.5)
         self.assertListEqual(p.set_values, [42, 43, 44, 44.5])
 
+        # Assert that stepping does not impact ``cache.set`` call, and that
+        # the value that is passed to ``cache.set`` call does not get
+        # propagated to parameter's ``set_cmd``
+        p.cache.set(40)
+        self.assertEqual(p.get_latest(), 40)
+        self.assertListEqual(p.set_values, [42, 43, 44, 44.5])
+
         # Test error conditions
         with self.assertLogs(level='WARN'):
             self.assertEqual(p.get_ramp_values("A", 1), [])
@@ -513,6 +567,69 @@ class TestParameter(TestCase):
         if isinstance(scales, Iterable) and not isinstance(offsets, Iterable):
             event('Scale is array but not offset')
 
+    @settings(max_examples=300)
+    @given(
+        values=iterable_or_number(
+            TestFloats, SharedSize, ValuesScalar, True),
+        offsets=iterable_or_number(
+            TestFloats, SharedSize, ValuesScalar, False),
+        scales=iterable_or_number(
+            TestFloats, SharedSize, ValuesScalar, False))
+    def test_scale_and_offset_raw_value_iterable_for_set_cache(
+            self, values, offsets, scales):
+        p = Parameter(name='test_scale_and_offset_raw_value', set_cmd=None)
+
+        # test that scale and offset does not change the default behaviour
+        p.cache.set(values)
+        self.assertEqual(p.raw_value, values)
+
+        # test setting scale and offset does not change anything
+        p.scale = scales
+        p.offset = offsets
+        self.assertEqual(p.raw_value, values)
+
+        np_values = np.array(values)
+        np_offsets = np.array(offsets)
+        np_scales = np.array(scales)
+        np_get_latest_values = np.array(p.get_latest())
+        # Without a call to ``get``, ``get_latest`` will just return old
+        # cached values without applying the set scale and offset
+        np.testing.assert_allclose(np_get_latest_values, np_values)
+        np_get_values = np.array(p.get())
+        # Now that ``get`` is called, the returned values are the result of
+        # application of the scale and offset. Obviously, calling
+        # ``get_latest`` now will also return the values with the applied
+        # scale and offset
+        np.testing.assert_allclose(np_get_values,
+                                   (np_values - np_offsets) / np_scales)
+        np_get_latest_values_after_get = np.array(p.get_latest())
+        np.testing.assert_allclose(np_get_latest_values_after_get,
+                                   (np_values - np_offsets) / np_scales)
+
+        # test ``cache.set`` for scalar values
+        if not isinstance(values, Iterable):
+            p.cache.set(values)
+            np.testing.assert_allclose(np.array(p.raw_value),
+                                       np_values * np_scales + np_offsets)
+            # No set/get cmd performed
+
+            # testing conversion back and forth
+            p.cache.set(values)
+            np_get_latest_values = np.array(p.get_latest())
+            # No set/get cmd performed
+            np.testing.assert_allclose(np_get_latest_values, np_values)
+
+        # adding statistics
+        if isinstance(offsets, Iterable):
+            event('Offset is array')
+        if isinstance(scales, Iterable):
+            event('Scale is array')
+        if isinstance(values, Iterable):
+            event('Value is array')
+        if isinstance(scales, Iterable) and isinstance(offsets, Iterable):
+            event('Scale is array and also offset')
+        if isinstance(scales, Iterable) and not isinstance(offsets, Iterable):
+            event('Scale is array but not offset')
 
     @given(scale=hst.integers(1, 100),
            value=hst.floats(min_value=1e-9, max_value=10))
@@ -653,6 +770,70 @@ class TestParameter(TestCase):
         assert a.get() == -10
 
 
+_P = Parameter
+
+
+@pytest.mark.parametrize(
+    argnames=('p', 'value', 'raw_value'),
+    argvalues=(
+        (_P('p', set_cmd=None, get_cmd=None), 4, 4),
+        (_P('p', set_cmd=False, get_cmd=None), 14, 14),
+        (_P('p', set_cmd=None, get_cmd=False), 14, 14),
+        (_P('p', set_cmd=None, get_cmd=None, vals=vals.OnOff()), 'on', 'on'),
+        (_P('p', set_cmd=None, get_cmd=None, val_mapping={'screw': 1}),
+         'screw', 1),
+        (_P('p', set_cmd=None, get_cmd=None, set_parser=str, get_parser=int),
+         14, '14'),
+        (_P('p', set_cmd=None, get_cmd=None, step=7), 14, 14),
+        (_P('p', set_cmd=None, get_cmd=None, offset=3), 14, 17),
+        (_P('p', set_cmd=None, get_cmd=None, scale=2), 14, 28),
+        (_P('p', set_cmd=None, get_cmd=None, offset=-3, scale=2), 14, 25),
+    ),
+    ids=(
+        'with_nothing_extra',
+        'without_set_cmd',
+        'without_get_cmd',
+        'with_on_off_validator',
+        'with_val_mapping',
+        'with_set_and_parsers',
+        'with_step',
+        'with_offset',
+        'with_scale',
+        'with_scale_and_offset',
+    )
+)
+def test_set_latest_works_for_plain_memory_parameter(p, value, raw_value):
+    # Set latest value of the parameter
+    p.cache.set(value)
+
+    # Assert the latest value and raw_value
+    assert p.get_latest() == value
+    assert p.raw_value == raw_value
+
+    # Assert latest value and raw_value via private attributes for strictness
+    assert p.cache._value == value
+    assert p.cache._raw_value == raw_value
+
+    # Now let's get the value of the parameter to ensure that the value that
+    # is set above gets picked up from the `_latest` dictionary (due to
+    # `get_cmd=None`)
+
+    if not hasattr(p, 'get'):
+        return  # finish the test here for non-gettable parameters
+
+    gotten_value = p.get()
+
+    assert gotten_value == value
+
+    # Assert the latest value and raw_value
+    assert p.get_latest() == value
+    assert p.raw_value == raw_value
+
+    # Assert latest value and raw_value via private attributes for strictness
+    assert p.cache._value == value
+    assert p.cache._raw_value == raw_value
+
+
 class TestValsandParseParameter(TestCase):
 
     def setUp(self):
@@ -689,7 +870,6 @@ class SimpleArrayParam(ArrayParameter):
 
     def get_raw(self):
         self._get_count += 1
-        self._save_val(self._return_val)
         return self._return_val
 
 
@@ -784,6 +964,15 @@ class TestArrayParameter(TestCase):
         self.assertTrue(hasattr(p, 'get'))
         self.assertFalse(hasattr(p, 'set'))
 
+        # Yet, it's possible to set the cached value
+        p.cache.set([6, 7, 8])
+        self.assertListEqual(p.get_latest(), [6, 7, 8])
+        # However, due to the implementation of this ``SimpleArrayParam``
+        # test parameter it's ``get`` call will return the originally passed
+        # list
+        self.assertListEqual(p.get(), [1, 2, 3])
+        self.assertListEqual(p.get_latest(), [1, 2, 3])
+
         with self.assertRaises(AttributeError):
             SettableArray([1, 2, 3], name, shape)
 
@@ -832,7 +1021,6 @@ class SimpleMultiParam(MultiParameter):
 
     def get_raw(self):
         self._get_count += 1
-        self._save_val(self._return_val)
         return self._return_val
 
 
@@ -845,7 +1033,7 @@ class SettableMulti(SimpleMultiParam):
 class TestMultiParameter(TestCase):
     def test_default_attributes(self):
         name = 'mixed_dimensions'
-        names = ['0D', '1D', '2D']
+        names = ('0D', '1D', '2D')
         shapes = ((), (3,), (2, 2))
         p = SimpleMultiParam([0, [1, 2, 3], [[4, 5], [6, 7]]],
                              name, names, shapes)
@@ -882,7 +1070,7 @@ class TestMultiParameter(TestCase):
 
     def test_explicit_attributes(self):
         name = 'mixed_dimensions'
-        names = ['0D', '1D', '2D']
+        names = ('0D', '1D', '2D')
         shapes = ((), (3,), (2, 2))
         labels = ['scalar', 'vector', 'matrix']
         units = ['V', 'A', 'W']
@@ -938,24 +1126,37 @@ class TestMultiParameter(TestCase):
         with self.assertRaises(AttributeError):
             MultiParameter(name, names, shapes)
 
-        p = SimpleMultiParam([0, [1, 2, 3], [[4, 5], [6, 7]]],
-                             name, names, shapes)
+        original_value = [0, [1, 2, 3], [[4, 5], [6, 7]]]
+        p = SimpleMultiParam(original_value, name, names, shapes)
 
         self.assertTrue(hasattr(p, 'get'))
         self.assertFalse(hasattr(p, 'set'))
+        # Ensure that ``cache.set`` works
+        new_cache = [10, [10, 20, 30], [[40, 50], [60, 70]]]
+        p.cache.set(new_cache)
+        self.assertListEqual(p.get_latest(), new_cache)
+        # However, due to the implementation of this ``SimpleMultiParam``
+        # test parameter it's ``get`` call will return the originally passed
+        # list
+        self.assertListEqual(p.get(), original_value)
+        self.assertListEqual(p.get_latest(), original_value)
+
         # We allow creation of Multiparameters with set to support
         # instruments that already make use of them.
-
         p = SettableMulti([0, [1, 2, 3], [[4, 5], [6, 7]]], name, names, shapes)
         self.assertTrue(hasattr(p, 'get'))
         self.assertTrue(hasattr(p, 'set'))
         value_to_set = [2, [1, 5, 2], [[8, 2], [4, 9]]]
         p.set(value_to_set)
         assert p.get() == value_to_set
+        # Also, ``cache.set`` works as expected
+        p.cache.set(new_cache)
+        assert p.get_latest() == new_cache
+        assert p.get() == value_to_set
 
     def test_full_name_s(self):
         name = 'mixed_dimensions'
-        names = ['0D', '1D', '2D']
+        names = ('0D', '1D', '2D')
         setpoint_names = ((),
                           ('setpoints_1D',),
                           ('setpoints_2D_1',
@@ -979,7 +1180,7 @@ class TestMultiParameter(TestCase):
         p._instrument = named_instrument
         self.assertEqual(str(p), 'astro_mixed_dimensions')
 
-        self.assertEqual(p.full_names, ['astro_0D', 'astro_1D', 'astro_2D'])
+        self.assertEqual(p.full_names, ('astro_0D', 'astro_1D', 'astro_2D'))
         self.assertEqual(p.setpoint_full_names,
                          ((), ('astro_setpoints_1D',),
                           ('astro_setpoints_2D_1', None)))
@@ -1042,6 +1243,14 @@ class TestStandardParam(TestCase):
         self.assertEqual(self._p, '5')
         self.assertEqual(p(), 5)
 
+        p.cache.set(7)
+        self.assertEqual(p.get_latest(), 7)
+        # Nothing has been passed to the "instrument" at ``cache.set``
+        # call, hence the following assertions should hold
+        self.assertEqual(self._p, '5')
+        self.assertEqual(p(), 5)
+        self.assertEqual(p.get_latest(), 5)
+
     def test_settable(self):
         p = Parameter('p', set_cmd=self.set_p, get_cmd=False)
 
@@ -1053,6 +1262,11 @@ class TestStandardParam(TestCase):
         self.assertTrue(hasattr(p, 'set'))
         self.assertFalse(hasattr(p, 'get'))
 
+        # For settable-only parameters, using ``cache.set`` may not make
+        # sense, nevertheless, it works
+        p.cache.set(7)
+        self.assertEqual(p.get_latest(), 7)
+
     def test_gettable(self):
         p = Parameter('p', get_cmd=self.get_p)
         self._p = 21
@@ -1066,6 +1280,14 @@ class TestStandardParam(TestCase):
         self.assertTrue(hasattr(p, 'get'))
         self.assertFalse(hasattr(p, 'set'))
 
+        p.cache.set(7)
+        self.assertEqual(p.get_latest(), 7)
+        # Nothing has been passed to the "instrument" at ``cache.set``
+        # call, hence the following assertions should hold
+        self.assertEqual(self._p, 21)
+        self.assertEqual(p(), 21)
+        self.assertEqual(p.get_latest(), 21)
+
     def test_val_mapping_basic(self):
         p = Parameter('p', set_cmd=self.set_p, get_cmd=self.get_p,
                       val_mapping={'off': 0, 'on': 1},
@@ -1087,6 +1309,16 @@ class TestStandardParam(TestCase):
         with self.assertRaises(KeyError):
             p()
 
+        self._p = 1  # for further testing
+
+        p.cache.set('off')
+        self.assertEqual(p.get_latest(), 'off')
+        # Nothing has been passed to the "instrument" at ``cache.set``
+        # call, hence the following assertions should hold
+        self.assertEqual(self._p, 1)
+        self.assertEqual(p(), 'on')
+        self.assertEqual(p.get_latest(), 'on')
+
     def test_val_mapping_with_parsers(self):
         # set_parser with val_mapping
         Parameter('p', set_cmd=self.set_p, get_cmd=self.get_p,
@@ -1106,6 +1338,14 @@ class TestStandardParam(TestCase):
         self._p = 'PVAL: 1'
         self.assertEqual(p(), 'on')
 
+        p.cache.set('off')
+        self.assertEqual(p.get_latest(), 'off')
+        # Nothing has been passed to the "instrument" at ``cache.set``
+        # call, hence the following assertions should hold
+        self.assertEqual(self._p, 'PVAL: 1')
+        self.assertEqual(p(), 'on')
+        self.assertEqual(p.get_latest(), 'on')
+
     def test_on_off_val_mapping(self):
         instrument_value_for_on = 'on_'
         instrument_value_for_off = 'off_'
@@ -1344,7 +1584,7 @@ class TestSetContextManager(TestCase):
         # A parameter that counts the number of times it has been set
         self.instrument.add_parameter("counting_parameter",
                                       set_cmd=self._cp_setter,
-                                      get_cmd=None)
+                                      get_cmd=self._cp_getter)
 
         # the mocked instrument state values of validated_param and
         # parsed_param
@@ -1353,6 +1593,7 @@ class TestSetContextManager(TestCase):
 
         # the counter value for counting_parameter
         self._cp_counter = 0
+        self._cp_get_counter = 0
 
     def _vp_getter(self):
         return self._vp_value
@@ -1369,14 +1610,82 @@ class TestSetContextManager(TestCase):
     def _cp_setter(self, value):
         self._cp_counter += 1
 
+    def _cp_getter(self):
+        self._cp_get_counter += 1
+        return self.instrument['counting_parameter'].cache._value
+
     def tearDown(self):
         self.instrument.close()
         del self.instrument
 
+    def test_set_to_none_when_parameter_is_not_captured_yet(self):
+        counting_parameter = self.instrument.counting_parameter
+        # Pre-conditions:
+        assert self._cp_counter == 0
+        assert self._cp_get_counter == 0
+        assert counting_parameter.cache._value is None
+        assert counting_parameter.get_latest.get_timestamp() is None
+
+        with counting_parameter.set_to(None):
+            # The value should not change
+            assert counting_parameter.cache._value is None
+            # The timestamp of the latest value should not be None anymore
+            assert counting_parameter.get_latest.get_timestamp() is not None
+            # Set method is not called
+            assert self._cp_counter == 0
+            # Get method is called once
+            assert self._cp_get_counter == 1
+
+        # The value should not change
+        assert counting_parameter.cache._value is None
+        # The timestamp of the latest value should still not be None
+        assert counting_parameter.get_latest.get_timestamp() is not None
+        # Set method is still not called
+        assert self._cp_counter == 0
+        # Get method is still called once
+        assert self._cp_get_counter == 1
+
+    def test_set_to_none_for_not_captured_parameter_but_instrument_has_value(self):
+        # representing instrument here
+        instr_value = 'something'
+        set_counter = 0
+
+        def set_instr_value(value):
+            nonlocal instr_value, set_counter
+            instr_value = value
+            set_counter += 1
+
+        # make a parameter that is linked to an instrument
+        p = Parameter('p', set_cmd=set_instr_value, get_cmd=lambda: instr_value,
+                      val_mapping={'foo': 'something', None: 'nothing'})
+
+        # pre-conditions
+        assert p.cache._value is None
+        assert p.cache._raw_value is None
+        assert p.cache.timestamp is None
+        assert set_counter == 0
+
+        with p.set_to(None):
+            # assertions after entering the context
+            assert set_counter == 1
+            assert instr_value == 'nothing'
+            assert p.cache._value is None
+            assert p.cache._raw_value == 'nothing'
+            assert p.cache.timestamp is not None
+
+        # assertions after exiting the context
+        assert set_counter == 2
+        assert instr_value == 'something'
+        assert p.cache._value == 'foo'
+        assert p.cache._raw_value == 'something'
+        assert p.cache.timestamp is not None
+
     def test_none_value(self):
         with self.instrument.a.set_to(3):
+            assert self.instrument.a.get_latest.get_timestamp() is not None
             assert self.instrument.a.get() == 3
         assert self.instrument.a.get() is None
+        assert self.instrument.a.get_latest.get_timestamp() is not None
 
     def test_context(self):
         self.instrument.a.set(2)
@@ -1386,7 +1695,7 @@ class TestSetContextManager(TestCase):
         assert self.instrument.a.get() == 2
 
     def test_validated_param(self):
-        assert self.instrument.parsed_param._latest['value'] is None
+        assert self.instrument.parsed_param.cache._value is None
         assert self.instrument.validated_param.get_latest() == "foo"
         with self.instrument.validated_param.set_to("bar"):
             assert self.instrument.validated_param.get() == "bar"
@@ -1394,7 +1703,7 @@ class TestSetContextManager(TestCase):
         assert self.instrument.validated_param.get() == "foo"
 
     def test_parsed_param(self):
-        assert self.instrument.parsed_param._latest['value'] is None
+        assert self.instrument.parsed_param.cache._value is None
         assert self.instrument.parsed_param.get_latest() == 42
         with self.instrument.parsed_param.set_to(1):
             assert self.instrument.parsed_param.get() == 1
@@ -1459,57 +1768,3 @@ def test_deprecated_param_warns():
     assert a.set_count == 1+9
     assert a.get() == 20
     assert a.get_count == 3
-
-
-def test_delegate_parameter_init():
-    """
-    Test that the lable and unit get used from source parameter if not
-    specified otherwise.
-    """
-    p = Parameter('testparam', set_cmd=None, get_cmd=None,
-                  label='Test Parameter', unit='V')
-    d = DelegateParameter('test_delegate_parameter', p)
-    assert d.label == p.label
-    assert d.unit == p.unit
-
-    d = DelegateParameter('test_delegate_parameter', p, unit='Ohm')
-    assert d.label == p.label
-    assert not d.unit == p.unit
-    assert d.unit == 'Ohm'
-
-
-def test_delegate_parameter_get_set_raises():
-    """
-    Test that providing a get/set_cmd kwarg raises an error.
-    """
-    p = Parameter('testparam', set_cmd=None, get_cmd=None)
-    for kwargs in ({'set_cmd': None}, {'get_cmd': None}):
-        with pytest.raises(KeyError) as e:
-            DelegateParameter('test_delegate_parameter', p, **kwargs)
-        assert str(e.value).startswith('\'It is not allowed to set')
-
-
-def test_delegate_parameter_scaling():
-    p = Parameter('testparam', set_cmd=None, get_cmd=None, offset=1, scale=2)
-    d = DelegateParameter('test_delegate_parameter', p, offset=3, scale=5)
-
-    p(1)
-    assert p.raw_value == 3
-    assert d() == (1-3)/5
-
-    d(2)
-    assert d.raw_value == 2*5+3
-    assert d.raw_value == p()
-
-
-def test_delegate_parameter_snapshot():
-    p = Parameter('testparam', set_cmd=None, get_cmd=None,
-                  offset=1, scale=2, initial_value=1)
-    d = DelegateParameter('test_delegate_parameter', p, offset=3, scale=5,
-                          initial_value=2)
-
-    snapshot = d.snapshot()
-    source_snapshot = snapshot.pop('source_parameter')
-    assert source_snapshot == p.snapshot()
-    assert snapshot['value'] == 2
-    assert source_snapshot['value'] == 13
